@@ -1,8 +1,8 @@
 ;;; action-lock.el --- invoke magic action by RET key on spell strings
 
-;; Copyright (c) 2002, 2003, 2004, 2005, 2006
+;; Copyright (c) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
 ;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;; $Id: action-lock.el,v 1.62 2006/03/22 13:29:25 hira Exp $
+;; $Id: action-lock.el,v 1.69 2009-02-05 15:00:56 hira Exp $
 ;;
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -51,6 +51,7 @@
   :group 'howm-faces)
 
 (defvar action-lock-magic-return-key "\C-m")
+(put 'action-lock-magic-return-key 'risky-local-variable t)
 (defvar action-lock-lighter " AL")
 (defvar action-lock-silent t
   "Inhibit font-lock-verbose if non-nil.")
@@ -80,6 +81,9 @@ Null prefix argument turns off the mode.
 (defvar action-lock-rules nil)
 (defvar action-lock-original-font-lock-keywords nil)
 (defvar action-lock-original-return nil)
+(put 'action-lock-rules 'risky-local-variable t)
+(put 'action-lock-original-font-lock-keywords 'risky-local-variable t)
+(put 'action-lock-original-return 'risky-local-variable t)
 
 (make-variable-buffer-local 'action-lock-rules)
 (make-variable-buffer-local 'action-lock-original-font-lock-keywords)
@@ -146,8 +150,8 @@ at the beginning of this file, when ARG-P is non-nil."
 
 ; (defun action-lock-escape-quote (s)
 ;   (apply 'concat
-; 	 (mapcar '(lambda (x) (if (string= x "'") "\\x27" x)) ;; for zsh
-; 		 (split-string s ""))))
+;        (mapcar '(lambda (x) (if (string= x "'") "\\x27" x)) ;; for zsh
+;                (split-string s ""))))
 
 ;; copied and modified from thingatpt.el [2004-01-30]
 (defvar action-lock-url-path-regexp
@@ -181,6 +185,7 @@ at the beginning of this file, when ARG-P is non-nil."
         (action-lock-browse action-lock-browse-regexp
                             action-lock-browse-regexp-pos) ;; http://...
         ))
+(put 'action-lock-default-rules 'risky-local-variable t)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; main
@@ -227,7 +232,7 @@ at the beginning of this file, when ARG-P is non-nil."
 (defun action-lock-rules ()
   action-lock-rules)
 (defun action-lock-set-rules (rules)
-  (setq action-lock-rules (howm-cl-remove-duplicates rules))
+  (setq action-lock-rules (howm-cl-remove-duplicates* rules))
 ;;   (message "Font lock...")
   (action-lock-font-lock)
 ;;   (message "...Done.")
@@ -291,38 +296,38 @@ at the beginning of this file, when ARG-P is non-nil."
 
 (defun action-lock-get-action/range ()
   (let* ((rules action-lock-rules)
-	 (current nil)
-	 (found nil))
+         (current nil)
+         (found nil))
     (while (and rules (not found))
       (save-excursion
-	(setq current (car rules)
-	      rules (cdr rules))
-	(let* ((regexp (car current))
+        (setq current (car rules)
+              rules (cdr rules))
+        (let* ((regexp (car current))
                (action (cadr current))
                (pos (caddr current))
                (range (action-lock-regexp-range regexp pos)))
-	  (if range
-	      (setq found (cons action range))))))
+          (if range
+              (setq found (cons action range))))))
     found))
 
 (defun action-lock-regexp-range (regexp &optional pos)
   (setq pos (or pos 0))
   (save-excursion
     (let ((c (point))
-	  (eol (line-end-position))
-	  (range nil)
- 	  (case-fold-search (if action-lock-use-case-fold-search
+          (eol (line-end-position))
+          (range nil)
+          (case-fold-search (if action-lock-use-case-fold-search
                                 action-lock-case-fold-search
                               case-fold-search))
-	  )
+          )
       (beginning-of-line)
       (while (and (<= (point) c)
-		  (re-search-forward regexp eol 'no-error)
-		  (not range))
+                  (re-search-forward regexp eol 'no-error)
+                  (not range))
         (let ((beg (match-beginning pos))
               (end (match-end pos)))
           (when (and (<= beg c) (< c end))
-	    (setq range (list beg end)))))
+            (setq range (list beg end)))))
       range)))
 
 (defun action-lock-regexp ()
