@@ -1,7 +1,7 @@
 ;;; howm-backend.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2005, 2006
+;;; Copyright (c) 2005, 2006, 2007, 2008, 2009
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-backend.el,v 1.23 2006/01/18 14:59:06 hira Exp $
+;;; $Id: howm-backend.el,v 1.41 2009-06-01 14:38:50 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -19,8 +19,8 @@
 ;;; USA.
 ;;--------------------------------------------------------------------
 
-(require 'howm-common)
-(require 'gfunc)
+(provide 'howm-backend)
+(require 'howm)
 
 ;; in preparation at now.
 ;; many WRONG COMMENTS and TENTATIVE CODES.
@@ -48,7 +48,7 @@
         ((eq folder 'buf) ':buf)
         ((listp folder) (car folder))))
 
-(defvar howm-folder-dispatchers (list #'howm-folder-type))
+(howm-defvar-risky howm-folder-dispatchers (list #'howm-folder-type))
 
 (gfunc-with howm-folder-dispatchers
   (gfunc-def howm-folder-items (folder &optional recursive-p)
@@ -83,9 +83,9 @@ Flag is non-nil if new page is created.")
        (string-match regexp (file-relative-name filename dir))))
 
 (defun howm-make-folder-from-items (items)
-  (howm-make-folder:pages (howm-cl-remove-duplicates (mapcar #'howm-item-page
-                                                             items)
-                                                     :test #'howm-page=)))
+  (howm-make-folder:pages (howm-cl-remove-duplicates* (mapcar #'howm-item-page
+                                                              items)
+                                                      :test #'howm-page=)))
 
 ;;;
 ;;; dir folder: single directory
@@ -188,7 +188,7 @@ DUMMY-EXCLUSION-CHECKER has no effect; it should be removed soon."
                                   (funcall searcher pages pattern fixed-p))))
                             howm-folder-grep-internal:pages-searcher))))
 
-(defvar howm-folder-grep-internal:pages-searcher
+(howm-defvar-risky howm-folder-grep-internal:pages-searcher
   '((:file . howm-folder-grep-internal:pages-files)
     (:buf  . howm-folder-grep-internal:pages-buffers)))
 (defun howm-folder-grep-internal:pages-files (pages pattern fixed-p)
@@ -233,11 +233,17 @@ DUMMY-EXCLUSION-CHECKER has no effect; it should be removed soon."
          (pages (mapcar (lambda (b) (howm-make-page:buf b)) bufs))
          (folder (howm-make-folder:pages pages)))
     (howm-view-directory folder)))
+(defvar howm-occur-force-fake-grep nil
+  "If non-nil, force `howm-occur' to use `howm-fake-grep'
+so that highlighting works correctly.")
 (defun howm-occur (regexp)
   (interactive "sSearch (regexp): ")
-  (howm-view-search-folder regexp
-                           (howm-make-folder:pages
-                            (list (howm-make-page:buf (current-buffer))))))
+  (let ((howm-view-use-grep (if howm-occur-force-fake-grep
+                                nil
+                              howm-view-use-grep)))
+    (howm-view-search-folder regexp
+                             (howm-make-folder:pages
+                              (list (howm-make-page:buf (current-buffer)))))))
 (defun howm-list-mark-ring ()
   (interactive)
   (let* ((page (howm-make-page:buf (current-buffer)))
@@ -249,7 +255,7 @@ DUMMY-EXCLUSION-CHECKER has no effect; it should be removed soon."
                                                  (e (line-end-position)))
                                              (buffer-substring b e)))))
                             (howm-make-item page summary place)))
-                        (howm-cl-remove-duplicates
+                        (howm-cl-remove-duplicates*
                          (cons (mark-marker) mark-ring)
                          :test #'howm-mark-same-line-p))))
     (howm-view-summary "<marks>" items)))
@@ -383,7 +389,7 @@ When that regexp is searched, hint strings are searched first for filtering."
            (files (if hints
                       (howm-rast-search (cadr folder) hints)
                     (howm-folder-files:rast folder))))
-      (setq files (howm-cl-remove-duplicates files :test #'string=))
+      (setq files (howm-cl-remove-duplicates* files :test #'string=))
       (howm-folder-grep-internal (howm-make-folder:files files)
                                  pattern-list fixed-p))))
 
@@ -507,7 +513,7 @@ ssearch: ")
   (howm-folder-grep-internal folder pattern fixed-p))
 
 (defvar *howm-view-force-case-fold-search* nil) ;; dirty!
-(defvar howm-view-grep-log-file nil)
+(howm-defvar-risky howm-view-grep-log-file nil)
 (defvar howm-view-grep-log-format "> %s | %s")
 
 (defun howm-grep-items (str file-list &optional fixed-p exclusion-checker)
@@ -548,7 +554,7 @@ ssearch: ")
 (defun howm-grep-multi-p ()
   howm-view-grep-file-stdin-option)
 
-;; will be obsolete when howm-real-grep-multi is stable
+;; obsolete
 (defun howm-real-grep-single (str file-list
                                   &optional fixed-p force-case-fold)
   "Call grep and parse its result.
@@ -560,46 +566,44 @@ ssearch: ")
       (error "Multiple patterns are not supported: %s" str)))
   (let ((grep-command (or (and fixed-p howm-view-fgrep-command)
                           howm-view-grep-command))
-        (opt (list howm-view-grep-option))
+        (opt (split-string howm-view-grep-option))
         (eopt (and howm-view-grep-expr-option
                    (list howm-view-grep-expr-option)))
         (case-fold (or force-case-fold
                        (not (let ((case-fold-search nil))
                               (string-match "[A-Z]" str))))))
-    (flet ((add-opt (pred x) (when (and pred x) (setq opt (cons x opt)))))
+    (labels ((add-opt (pred x) (when (and pred x) (setq opt (cons x opt)))))
       (add-opt case-fold howm-view-grep-ignore-case-option)
       (add-opt fixed-p howm-view-grep-fixed-option)
       (add-opt (not fixed-p) howm-view-grep-extended-option))
     (with-temp-buffer
-      (let* ((fs (mapcar #'expand-file-name file-list))
-             (lines (howm-call-process grep-command
-                                       `(,@opt ,@eopt ,str ,@fs)))
+      (let* ((fs (howm-expand-file-names file-list))
+             (lines (howm-call-process* grep-command
+                                        `(,@opt ,@eopt ,str) fs))
              (parsed (mapcar 'howm-grep-parse-line lines)))
         (remove nil parsed)))))
 
-;; experimental [2004-04-11]
-(defun howm-real-grep-multi (str file-list
-                                      &optional fixed-p force-case-fold)
+(defun howm-real-grep-multi (str file-list &optional fixed-p force-case-fold)
   (let ((grep-command (or (and fixed-p howm-view-fgrep-command)
                           howm-view-grep-command))
-        (opt (list howm-view-grep-option))
+        (opt (split-string howm-view-grep-option))
         (eopt (split-string howm-view-grep-file-stdin-option)))
     (let* ((str-list (cond ((stringp str) (list str))
                            ((listp str) str)
                            (t (error "Wrong type: %s" str))))
            (caps-p (howm-cl-member-if (lambda (s) (howm-capital-p s)) str-list))
            (case-fold (or force-case-fold (not caps-p))))
-      (flet ((add-opt (pred x) (when (and pred x) (setq opt (cons x opt)))))
+      (labels ((add-opt (pred x) (when (and pred x) (setq opt (cons x opt)))))
         (add-opt case-fold howm-view-grep-ignore-case-option)
         (add-opt fixed-p howm-view-grep-fixed-option)
         (add-opt (not fixed-p) howm-view-grep-extended-option))
       (with-temp-buffer
-        (let* ((fs (mapcar #'expand-file-name file-list))
+        (let* ((fs (howm-expand-file-names file-list))
                (pat (apply #'concat
                            (mapcar (lambda (s) (concat s "\n")) str-list)))
-               (lines (howm-call-process grep-command
-                                              `(,@opt ,@eopt ,@fs)
-                                              nil pat))
+               (lines (howm-call-process* grep-command
+                                          `(,@opt ,@eopt) fs
+                                          nil pat))
                (parsed (mapcar 'howm-grep-parse-line lines)))
           (remove nil parsed))))))
 
@@ -684,9 +688,10 @@ STR can be list of strings. They are regarded as 'or' pattern of all elements."
 (defun howm-page-type (page &rest r)
   (cond ((stringp page) ':file)
         ((bufferp page) ':buf)
+        ((null page) ':nil)
         ((listp page) (car page))))
 
-(defvar howm-page-dispatchers (list #'howm-page-type))
+(howm-defvar-risky howm-page-dispatchers (list #'howm-page-type))
 
 (gfunc-with howm-page-dispatchers
   (gfunc-def howm-page-name   (page))
@@ -705,6 +710,24 @@ STR can be list of strings. They are regarded as 'or' pattern of all elements."
 
 (defalias 'howm-save-buffer #'save-buffer)
 
+(defun howm-insert-buffer-contents (buffer)
+  (insert (with-current-buffer buffer
+            (save-restriction
+              (widen)
+              (let ((limit (point-max)))
+                (when howm-view-contents-limit
+                  (setq limit (min limit howm-view-contents-limit)))
+                (buffer-substring-no-properties (point-min) limit))))))
+
+(defun howm-insert-file-contents (file)
+  (insert-file-contents file nil nil howm-view-contents-limit))
+
+;; (defun howm-page-insert-range ()
+;;   (let ((limit (point-max)))
+;;     (when howm-view-contents-limit
+;;       (setq limit (min limit howm-view-contents-limit)))
+;;     (list (point-min) limit)))
+
 ;; (defun howm-page-save (&optional args)
 ;;   (interactive "p")
 ;;   (with-current-buffer (get-file-buffer (howm-page-name howm-buffer-page))
@@ -731,19 +754,17 @@ STR can be list of strings. They are regarded as 'or' pattern of all elements."
 
 (defun howm-page-open:file (page)
   (find-file (howm-page-name page))
-;;   (set (make-local-variable 'howm-buffer-page page))
-  )
+  ;; widen is desired when corresponding file is already opened and
+  ;; its buffer is narrowed.
+  (widen))
 
 (defun howm-page-insert:file (page)
   (let ((b (get-file-buffer page)))
     (if (and b
              howm-view-watch-modified-buffer
              (not howm-view-use-grep))
-        (insert (with-current-buffer b
-                  (save-restriction
-                    (widen)
-                    (buffer-substring-no-properties (point-min) (point-max)))))
-      (insert-file-contents page))))
+        (howm-insert-buffer-contents b)
+      (howm-insert-file-contents page))))
 
 (defun howm-page-viewer:file (page)
   (let* ((ls (lambda (dir)
@@ -783,15 +804,18 @@ STR can be list of strings. They are regarded as 'or' pattern of all elements."
 (defun howm-page-name:buf (page)
   (buffer-name page))
 
+(defconst howm-dummy-mtime (encode-time 0 0 9 1 1 1970)
+  "Dummy mtime which has no meaning.")
+
 (defun howm-page-mtime:buf (page)
-  (encode-time 0 0 9 1 1 1970))  ;; no meaning
+  howm-dummy-mtime)
 
 (defun howm-page-open:buf (page)
   (switch-to-buffer page))
 
 (defun howm-page-insert:buf (page)
   (when (not (howm-buffer-killed-p page))
-    (insert-buffer-substring page)))
+    (howm-insert-buffer-contents page)))
 
 (defun howm-page-viewer:buf (page)
   nil)
@@ -801,6 +825,34 @@ STR can be list of strings. They are regarded as 'or' pattern of all elements."
   (when (buffer-file-name page)
     (howm-set-configuration-for-file-name (buffer-file-name page))))
 
+;;;
+;;; nil page: dummy page
+;;;
+
+(defun howm-make-page:nil ()
+  nil)
+
+(defun howm-page-name:nil (page)
+  "")
+
+(defun howm-page-mtime:nil (page)
+  howm-dummy-mtime)
+
+(defun howm-page-open:nil (page)
+  "Do nothing."
+  nil)
+
+(defun howm-page-insert:nil (page)
+  "Do nothing."
+  nil)
+
+(defun howm-page-viewer:nil (page)
+  nil)
+
+(defun howm-page-set-configuration:nil (page)
+  "Do nothing."
+  nil)
+
 ;;;
 ;;; rot13file page: almost same as file except that it is rot13ed
 ;;;
@@ -872,7 +924,7 @@ STR can be list of strings. They are regarded as 'or' pattern of all elements."
         ((functionp viewer) ':func0)
         ((listp viewer)     ':func)))
 
-(defvar howm-viewer-dispatchers (list #'howm-viewer-type))
+(howm-defvar-risky howm-viewer-dispatchers (list #'howm-viewer-type))
 
 (gfunc-with howm-viewer-dispatchers
   (gfunc-def howm-viewer-call      (viewer page))
@@ -935,7 +987,7 @@ STR can be list of strings. They are regarded as 'or' pattern of all elements."
 
 (defun howm-make-item (page &optional summary place offset home privilege)
   (list page summary place offset home privilege))
-(defun howm-item-page      (item) (howm-item-nth 0 item ""))
+(defun howm-item-page      (item) (nth 0 item)) ;; page can be nil.
 (defun howm-item-summary   (item) (howm-item-nth 1 item ""))
 (defun howm-item-place     (item) (howm-item-nth 2 item nil))
 (defun howm-item-offset    (item) (howm-item-nth 3 item nil))
@@ -957,6 +1009,8 @@ STR can be list of strings. They are regarded as 'or' pattern of all elements."
 (defun howm-item-name (item)
   (format "%s" (howm-page-name (howm-item-page item))))
 
+(defun howm-item-dup (item) (mapcar #'identity item))
+
 ;; For backward compatibility. Don't use them any more.
 ;; ;; item = (filename summary place offset home)
 (defun howm-view-make-item (filename &rest r)
@@ -977,7 +1031,7 @@ STR can be list of strings. They are regarded as 'or' pattern of all elements."
 
 ;; historical & awkward mechanism
 
-(defvar howm-search-path nil)
+(howm-defvar-risky howm-search-path nil)
 (defvar howm-search-other-dir nil)
 (defvar *howm-independent-directories* nil) ;; for internal use
 
@@ -1010,17 +1064,12 @@ With arg, search `howm-search-path' iff arg is positive."
           (not howm-search-other-dir)))
   (message "howm search-path = %s" (howm-search-path)))
 
-(defvar howm-keyword-buffer-name-format " *howm-keys:%s*")
-(defun howm-get-keyword-buffer (&optional dir)
-  (setq dir (expand-file-name dir))
-  (get-buffer-create (format howm-keyword-buffer-name-format dir)))
 (defun howm-open-directory-independently (dir)
   (interactive "DDirectory: ")
   (add-to-list '*howm-independent-directories*
                (expand-file-name dir))
   (let ((default-directory dir))
-    (howm-view-directory dir t))
-  (howm-list-normalize)
+    (howm-normalize-show "" (howm-folder-items dir t)))
   (let ((files (mapcar #'howm-view-item-filename (howm-view-item-list))))
     (with-temp-buffer
       (setq default-directory dir)
@@ -1031,15 +1080,13 @@ With arg, search `howm-search-path' iff arg is positive."
               (howm-keyword-add-current-buffer))
             files))))
 
+(defvar howm-keyword-buffer-name-format " *howm-keys:%s*")
 (defun howm-keyword-buffer ()
-  (let ((d (howm-independent-search-path)))
-    (if d
-        (howm-get-keyword-buffer d)
-      (find-file-noselect (howm-keyword-file)))))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; provide
-
-(provide 'howm-backend)
+  (let* ((dir (howm-independent-search-path))
+         (buffer-name (format howm-keyword-buffer-name-format
+                              (if dir (expand-file-name dir) ""))))
+    (if dir
+        (get-buffer-create buffer-name)
+      (howm-get-buffer-for-file (howm-keyword-file) buffer-name))))
 
 ;;; howm-backend.el ends here
