@@ -1,7 +1,7 @@
 ;;; howm-common.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2002, 2003, 2004, 2005, 2006
+;;; Copyright (c) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-common.el,v 1.72 2006/05/15 13:34:32 hira Exp $
+;;; $Id: howm-common.el,v 1.84 2009-06-01 14:38:50 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 ;;; USA.
 ;;--------------------------------------------------------------------
 
+(require 'howm-cl)
 (require 'howm-vars)
 
 (defun howm-sort (evaluator comparer obj-list)
@@ -54,6 +55,10 @@ When TARGET and DIR are same, (not STRICT) is returned."
         (t howm-abbreviate-file-name)))
 ;; (defun howm-abbreviate-file-name-p () howm-abbreviate-file-name)
 
+(defun howm-expand-file-names (file-list)
+  (mapcar (lambda (f) (directory-file-name (expand-file-name f)))
+          file-list))
+
 (defun howm-fontify (&optional i-dont-understand-this)
   (cheat-font-lock-fontify i-dont-understand-this))
 
@@ -264,6 +269,15 @@ If PASS-RET-THROUGH is non-nil, RET is unread and nil is returned.
 ;;      (set-buffer-modified-p nil)
 ;;      (setq buffer-read-only t)))
 
+(defun howm-get-buffer-for-file (file &optional buffer-name)
+  "Get buffer for FILE, and rename buffer if BUFFER-NAME is given."
+  (let ((buf (find-file-noselect file)))
+    (when buffer-name
+      (with-current-buffer buf
+        (rename-buffer buffer-name)))
+    buf))
+
+(defvar howm-log-buffer-name-format " *howm-log:%s*")
 (defun howm-write-log (message fmt file &optional limit remove-fmt)
   "Write MESSAGE with format FMT to the top of FILE.
 FORMAT must have two %s; the formar is time and the latter is message.
@@ -271,22 +285,24 @@ When LIMIT is non-nil, only that number of recent messages are recorded.
 When REMOVE-FMT is non-nil, it is used to generate regular expression
 to remove matched lines. REMOVE-FMT must have one %s."
   (save-excursion
-    (with-current-buffer (find-file-noselect file)
-      (goto-char (point-min))
-      (when remove-fmt
-        (save-excursion
-          (flush-lines (format remove-fmt (regexp-quote message)))))
-      (insert (format fmt
-                      (format-time-string howm-dtime-format (current-time))
-                      message)
-              "\n")
-      (when limit
-        ;; Since I don't understand `selective-display' in goto-line,
-        ;; I avoid it.
+    (let ((buffer-name (format howm-log-buffer-name-format
+                               (file-name-nondirectory file))))
+      (with-current-buffer (howm-get-buffer-for-file file buffer-name)
         (goto-char (point-min))
-        (when (= (forward-line limit) 0) ;; buffer lines > LIMIT
-          (delete-region (point) (point-max))))
-      (howm-basic-save-buffer))))
+        (when remove-fmt
+          (save-excursion
+            (flush-lines (format remove-fmt (regexp-quote message)))))
+        (insert (format fmt
+                        (format-time-string howm-dtime-format (current-time))
+                        message)
+                "\n")
+        (when limit
+          ;; Since I don't understand `selective-display' in goto-line,
+          ;; I avoid it.
+          (goto-char (point-min))
+          (when (= (forward-line limit) 0) ;; buffer lines > LIMIT
+            (delete-region (point) (point-max))))
+        (howm-basic-save-buffer)))))
 
 (defun howm-get-symbol (soft &rest args)
   "Return the canonical symbol for a specified name.
@@ -327,6 +343,25 @@ returns ((\"a\" \"aaa\" \"abc\") (\"x\" \"xyz\"))."
                        ans)))))
 ;; (howm-classify (lambda (s) (substring s 0 1)) '("aaa" "abc" "xyz"))
 
+(defun howm-message-nolog (str &rest args)
+  (let ((message-log-max nil))
+    (apply #'message `(,str ,@args))))
+
+(defun howm-decode-time (&optional specified-time)
+  "Decode SPECIFIED-TIME and remove DOW, DST, ZONE.
+When we do something like (encode-time (decode-time)), we use this function
+instead of the original `decode-time', so that we can force
+current timezone rule uniformly to avoid inconsistency."
+  (butlast (decode-time specified-time) 3))
+
+(put 'howm-with-need 'lisp-indent-hook 0)
+(defmacro howm-with-need (&rest body)
+  "Execute BODY where (need xxx) exits from this form if xxx is nil."
+  (let ((g (howm-cl-gensym)))
+    `(catch ',g
+       (labels ((need (x) (or x (throw ',g nil))))
+         ,@body))))
+
 ;; view-in-background
 
 (defvar *howm-view-in-background* nil
@@ -336,6 +371,7 @@ Use `howm-view-in-background' and `howm-view-in-background-p' instead.")
 
 (put 'howm-view-in-background 'lisp-indent-hook 0)
 (defmacro howm-view-in-background (&rest body)
+  "Obsolete. Do not use this any more."
   `(let ((*howm-view-in-background* t))
      ,@body))
 
@@ -358,21 +394,6 @@ Use `howm-view-in-background' and `howm-view-in-background-p' instead.")
                                                  howm-excluded-file-regexp
                                                  filename))
         (howm-search-path))))
-;; (defun howm-exclude-p (filename)
-;;   (string-match howm-excluded-file-regexp filename))
-
-;; These functions should be removed.
-(defun howm-item-exclude-p (item) nil)
-(defun howm-list-exclude () nil)
-(defun howm-view-item-exclude-p (name-checker item) nil)
-
-;; (defun howm-item-exclude-p (item)
-;;   (howm-view-item-exclude-p #'howm-exclude-p item))
-;; (defun howm-list-exclude ()
-;;   (howm-view-filter-general #'howm-item-exclude-p t))
-;; (defun howm-view-item-exclude-p (name-checker item)
-;;   (and (not (howm-view-item-privilege item))
-;;        (funcall name-checker (howm-view-item-filename item))))
 
 ;;; history of search
 
@@ -388,6 +409,8 @@ Use `howm-view-in-background' and `howm-view-in-background-p' instead.")
 
 (defun howm-history ()
   (interactive)
+  (unless (file-exists-p howm-history-file)
+    (error "No history."))
   ;; disable expansion of %schedule etc.
   (let ((howm-menu-display-rules nil)) ;; dirty
     (howm-menu-open howm-history-file)))
@@ -418,7 +441,7 @@ examples:
  (howm-with-coding-system 'euc-jp-unix ...)  ;; for both read and write
  (howm-with-coding-system '(utf-8-unix . sjis-unix) ...)  ;; (read . write)
  (howm-with-coding-system nil ...)  ;; howm-process-coding-system is used."
-  (let ((g (gensym))
+  (let ((g (howm-cl-gensym))
         (cs (or coding-system 'howm-process-coding-system)))
     `(let* ((,g ,cs)
             (coding-system-for-read  (or (car-safe ,g) ,g))
@@ -454,6 +477,39 @@ examples:
       (set-buffer buf))
     (split-string (buffer-substring (point-min) (point-max)) "\n")))
 
+(defun howm-call-process* (command common-args rest-args &rest options)
+  ;; (howm-call-process* "grep" '("pattern") '("001" ... "999"))
+  ;; is expanded to concatenation of
+  ;; (howm-call-process "grep" '("pattern" "001" ... "099"))
+  ;; (howm-call-process "grep" '("pattern" "101" ... "199"))
+  ;; ..., depending on howm-command-length-limit.
+  (labels ((div (a limit measure)
+                ;; (div '(3 1 4 1 5 9 2 6 5 3 5 8 9 7 9 3 2 3 8) 20 #'identity)
+                ;; ==> ((3 1 4 1 5) (9 2 6) (5 3 5) (8 9) (7 9 3) (2 3 8))
+                ;; [create new group when sum >= 20]
+                (let ((sum limit) ;; measure >= 0 is assumed.
+                      (ans nil))
+                  (mapc (lambda (x)
+                          (let* ((w (funcall measure x))
+                                 (new-sum (+ sum w)))
+                            (if (< new-sum limit)
+                                (setq sum new-sum
+                                      ans (cons (cons x (car ans)) (cdr ans)))
+                              (setq sum w
+                                    ans (cons (list x) ans)))))
+                        a)
+                  (reverse (mapcar #'reverse ans)))))
+    ;; XEmacs 21.4 lacks "string-bytes".
+    (let* ((len (symbol-function
+                 (howm-cl-find-if #'fboundp '(string-bytes length))))
+           (limit (apply #'- howm-command-length-limit
+                         (mapcar len (cons command common-args))))
+           (as (div rest-args limit len)))
+      (mapcan (lambda (args)
+                (apply #'howm-call-process
+                       command (append common-args args) options))
+              as))))
+
 ;;; schedule-interval & reminder-setting (clean me)
 
 (defvar howm-reminder-schedule-interval nil
