@@ -1,7 +1,7 @@
 ;;; howm-menu.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2002, 2003, 2004, 2005, 2006
+;;; Copyright (c) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-menu.el,v 1.85 2006/02/01 15:50:07 hira Exp $
+;;; $Id: howm-menu.el,v 1.99 2009-02-05 15:00:55 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -19,14 +19,15 @@
 ;;; USA.
 ;;;--------------------------------------------------------------------
 
-(require 'howm-common)
+(provide 'howm-menu)
+(require 'howm)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; customize
 
 ;;; general
 
-(defvar howm-menu-mode-map nil)
+(howm-defvar-risky howm-menu-mode-map nil)
 (let ((m (make-keymap)))
   (define-key m action-lock-magic-return-key 'howm-menu-invoke)
   (define-key m [tab] 'action-lock-goto-next-link)
@@ -49,10 +50,18 @@
 (when howm-randomize
   (random t))
 
-(defvar howm-menu-list-format "> %s | %s"
-  "Format to show schedule/todo/recent/random list in `howm-menu-mode'.")
-(defvar howm-menu-list-regexp "^\\(>\\)\\([^|\r\n]*|\\) +\\(.*\\)$"
-  "Regexp to find and parse schedule/todo/recent/random list in `howm-menu-mode'.")
+(defvar howm-menu-reminder-format "> %s | %s"
+  "Format to show schedule/todo list in `howm-menu-mode'.")
+(defvar howm-menu-list-format
+  (let* ((path (format-time-string howm-file-name-format))
+         (width (length (file-name-sans-extension
+                         (file-name-nondirectory path)))))
+    (concat "> %-" (format "%s" width) "s | %s"))
+  "Format to show recent/random list in `howm-menu-mode'.")
+(defvar howm-menu-list-regexp "^\\(>\\([^|\r\n]*|\\)\\) +\\(.*\\)$"
+  "Regexp to find and parse schedule/todo/recent/random list in `howm-menu-mode'.
+`howm-menu-list-regexp-action-pos' must cover header part.
+Otherwise, `howm-action-lock-forward' may be invoked unintentionally.")
 (defvar howm-menu-list-regexp-key-pos 3
   "Position of target string for action-lock in history buffer.
 This target is searched when action-lock is invoked.")
@@ -74,7 +83,7 @@ in `howm-menu-mode'.")
 
 ;;; dynamic contents
 
-(defvar howm-menu-allow
+(howm-defvar-risky howm-menu-allow
   '(howm-menu-schedule
     howm-menu-todo
     howm-menu-reminder
@@ -84,7 +93,7 @@ in `howm-menu-mode'.")
     howm-menu-categorized-reminder
     ))
 
-(defvar howm-menu-display-rules
+(howm-defvar-risky howm-menu-display-rules
   `(
     ;; static
     ("%sdays"    . "%here%howm-menu-schedule-days")
@@ -115,22 +124,22 @@ Regexp R1 is replaced by T1 if T1 is a string.
 ;;   'previous: previous non-menu buffer (switch-to-buffer)
 ;;   'current: current menu buffer
 
-(defvar howm-menu-command-table-common
+(howm-defvar-risky howm-menu-command-table-common
   '(
     (("%eval%\\(.*$\\)" 1) howm-menu-eval previous)
     (("%call%\\(.*$\\)" 1) howm-menu-call previous)
      ))
 
-(defvar howm-menu-action-arg 'howm-menu-action-arg-name)
+(howm-defvar-risky howm-menu-action-arg 'howm-menu-action-arg-name)
 
 ;;; which is opened as menu?
 
-(defvar howm-menu-keyword-regexp "^%.*%$")
-(defvar howm-menu-top "%menu%")
+(howm-defvar-risky howm-menu-keyword-regexp "^%.*%$")
+(howm-defvar-risky howm-menu-top "%menu%")
 
 ;;; misc.
 
-(defvar howm-menu-toggle-invisible "%|")
+(howm-defvar-risky howm-menu-toggle-invisible "%|")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; internal
@@ -147,7 +156,7 @@ Regexp R1 is replaced by T1 if T1 is a string.
 (defvar howm-menu-last-time (current-time))
 (defvar howm-menu-buffer-file nil)
 (defvar howm-menu-buffer-file-place nil)
-(defvar howm-menu-mode-local-map nil)
+(howm-defvar-risky howm-menu-mode-local-map nil)
 (make-variable-buffer-local 'howm-menu-previous-buffer)
 (make-variable-buffer-local 'howm-menu-next-expiry-time)
 (make-variable-buffer-local 'howm-menu-last-time)
@@ -230,9 +239,9 @@ key	binding
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; refresh
 
-(defvar howm-menu-shortcut-assoc nil)
+(howm-defvar-risky howm-menu-shortcut-assoc nil)
 (make-variable-buffer-local 'howm-menu-shortcut-assoc)
-(defvar howm-menu-invisible t
+(howm-defvar-risky howm-menu-invisible t
   "*Non nil if 'invisible' property should be used in menu.
 This must be t at now.
 When this is nil, delete-region is used instead, and bug appears.")
@@ -292,8 +301,7 @@ When this is nil, delete-region is used instead, and bug appears.")
       (howm-mode 1))))
 
 (defun howm-menu-set-face ()
-  (make-local-variable 'font-lock-keywords-only)
-  (setq font-lock-keywords-only t)
+  (set (make-local-variable 'font-lock-keywords-only) t)
   (howm-menu-add-font-lock)
   (font-lock-fontify-buffer)
   (when howm-menu-toggle-invisible
@@ -314,6 +322,7 @@ When this is nil, delete-region is used instead, and bug appears.")
         footer)))
 
 (defun howm-menu-refresh-background ()
+  ;; save-current-buffer doesn't work on GNU Emacs 21.4.1
   (let ((b (current-buffer)))
     (howm-menu t)
     (switch-to-buffer b)))
@@ -335,17 +344,17 @@ When this is nil, delete-region is used instead, and bug appears.")
   (let* ((d action-lock-default-rules)
          (f (howm-action-lock-reminder-forward-rules))
          (j (howm-menu-list-rules))
-        (m (mapcar (lambda (pair)
-                     (let* ((h (car pair))
-                            (r (if (listp h) (car h) h))
-                            (n (if (listp h) (second h) nil))
-                            (args (if n
-                                     `(list (match-string-no-properties ,n))
-                                    nil))
-                            (functab (cdr pair))
-                            (c (howm-menu-action functab args)))
-                       (list r c)))
-                   (howm-menu-command-table))))
+         (m (mapcar (lambda (pair)
+                      (let* ((h (car pair))
+                             (r (if (listp h) (car h) h))
+                             (n (if (listp h) (second h) nil))
+                             (args (if n
+                                       `(list (match-string-no-properties ,n))
+                                     nil))
+                             (functab (cdr pair))
+                             (c (howm-menu-action functab args)))
+                        (list r c)))
+                    (howm-menu-command-table))))
     (append m d j f)))
 
 ;; Elisp is not Scheme. Lambda is not closure. Don't forget dynamic binding.
@@ -392,7 +401,7 @@ When this is nil, delete-region is used instead, and bug appears.")
 
 (defun howm-menu-shortcut-sort (keys)
   (mapconcat #'identity
-             (sort (copy-list keys) #'string<)
+             (sort (copy-sequence keys) #'string<)
              ""))
 
 (defun howm-menu-shortcut-warn ()
@@ -402,7 +411,7 @@ When this is nil, delete-region is used instead, and bug appears.")
              (howm-menu-shortcut-sort *howm-menu-shortcut-multidef-keys*)
              (howm-menu-shortcut-sort *howm-menu-shortcut-keys*))))
 
-;; Check howm-menu-mode-local-map for debug
+;; Check howm-menu-mode-local-map if you want to debug howm-menu-shortcut.
 (defun howm-menu-shortcut ()
   (let* ((beg (match-beginning 0))
          (end (match-end 0))
@@ -554,10 +563,21 @@ When this is nil, delete-region is used instead, and bug appears.")
         (forward-line)))))
 
 (defun howm-menu-font-lock-rules ()
-  (list `(,howm-menu-key-regexp
-          (,howm-menu-key-regexp-key-pos howm-menu-key-face t))
-        `(,howm-menu-list-regexp
-          (,howm-menu-list-regexp-face-pos howm-menu-list-face t))))
+  `((,howm-menu-key-regexp
+     (,howm-menu-key-regexp-key-pos howm-menu-key-face t))
+    ;; In menu-list form "> FILE-NAME | ",
+    ;; I want to hide annoying long underlines drawn by action-lock.
+    (,howm-menu-list-regexp
+     (,howm-menu-list-regexp-face-pos howm-menu-list-face t))
+    ;; But some users may want to highlight today's YYYY-MM-DD even if
+    ;; it is a part of a FILE-NAME.
+    ;; The next code makes duplicated entries; they are already put into
+    ;; font-lock-keywords by howm-reminder-add-font-lock
+    ;; in howm-initialize-buffer because menu is howm-mode.
+    ;; They are hidden by the above rule in FILE-NAME columns,
+    ;; and I need to put them again now. Sigh...
+    ;; Clean me!
+    ,@(howm-reminder-today-font-lock-keywords)))
 (defun howm-menu-add-font-lock ()
   (cheat-font-lock-append-keywords (howm-menu-font-lock-rules)))
 
@@ -645,7 +665,7 @@ When this is nil, delete-region is used instead, and bug appears.")
   "Generate output string for items in howm menu.
 LABEL is only used for message.
 FORMATTER is a function which receives an item and returns an output string
-(without newline).
+ (without newline).
 FORMATTER can be nil for standard style, 'todo for todo style,
 or 'schedule for schedule style.
 ITEM-LIST is list of items which should be shown."
@@ -664,8 +684,9 @@ ITEM-LIST is list of items which should be shown."
 ;;; schedule/todo
 
 (defun howm-menu-format-todo (item)
-  (if (stringp item) ;; item can be a separator string
-      item
+  ;; item can be a separator.
+  (if (eq (howm-page-type (howm-item-page item)) ':nil)
+      (howm-item-summary item)
     (let ((dow-str (cond (howm-menu-todo-show-day-of-week nil)
                          (t "  "))))
       (howm-menu-format-reminder item dow-str t))))
@@ -682,12 +703,16 @@ ITEM-LIST is list of items which should be shown."
                                (howm-todo-priority item))
                      ""))
          (h (format "%s%3s%s" dow-str late priority)))
-    (howm-menu-list-format h (howm-view-item-summary item) item)))
+    (howm-menu-list-format h (howm-view-item-summary item) item
+                           howm-menu-reminder-format)))
 
 (defun howm-day-of-week-string (&optional day-of-week)
   ;; 0 = Sunday
-  (let ((dow (or day-of-week (nth 6 (decode-time)))))
-    (substring (howm-day-of-week) dow (1+ dow))))
+  (let ((dow (or day-of-week (nth 6 (decode-time))))
+        (names (howm-day-of-week)))
+    (cond ((stringp names) (substring names dow (1+ dow))) ;; backward compatibility
+          ((listp names) (nth dow names))
+          (t "  "))))
 
 ;;; recent/random
 
@@ -698,31 +723,21 @@ ITEM-LIST is list of items which should be shown."
          ;; Unique name is needed for dynamic binding. Sigh...
          (h-r-m-evaluator (if random
                               (lambda (f) (number-to-string (random)))
-                            (lambda (f) (howm-view-xtime f 'm))))
+                            #'howm-view-mtime))
          (sorted (howm-sort (lambda (f) (funcall h-r-m-evaluator f))
                             #'howm-view-string>
                             (mapcar #'howm-item-name
                                     (howm-folder-items howm-directory t))))
-         (files (howm-first-n sorted num))
-         (items (howm-view-search-items (howm-menu-recent-regexp)
-                                        files summarizer)))
-    (howm-first-n items num)))
-
-;; (defun howm-recent-menu (num &optional random)
-;;   (let* ((summarizer #'(lambda (file line content) content))
-;;          ;; Unique name is needed for dynamic binding. Sigh...
-;;          (h-r-m-evaluator (if random
-;;                               (lambda (f) (number-to-string (random)))
-;;                             (lambda (f) (howm-view-xtime f 'm))))
-;;          (sorted (howm-sort (lambda (f) (funcall h-r-m-evaluator f))
-;;                             #'howm-view-string>
-;;                             (howm-files-in-directory howm-directory
-;;                                                      'dummy)))
-;; ;;                                                      #'howm-exclude-p)))
-;;          (files (howm-first-n sorted num))
-;;          (items (howm-view-search-items (howm-menu-recent-regexp)
-;;                                         files summarizer)))
-;;     (howm-first-n items num)))
+         (files (howm-first-n sorted num)))
+    (let ((r (howm-menu-recent-regexp)))
+      (if random
+          (howm-cl-mapcan (lambda (f)
+                            (let ((is (howm-view-search-items r (list f)
+                                                              summarizer)))
+                              (and is (list (nth (random (length is))
+                                                 is)))))
+                          files)
+        (howm-first-n (howm-view-search-items r files summarizer) num)))))
 
 (defun howm-menu-recent-regexp ()
   (or howm-menu-recent-regexp (howm-view-title-regexp-grep)))
@@ -744,13 +759,13 @@ ITEM-LIST is list of items which should be shown."
           (keyword (howm-keyword-search keyword)) ;; history
           (t (error "Target is not specified."))))) ;; can't happen
 
-(defun howm-menu-format-item (item)
+(defun howm-menu-format-item (item &optional list-format)
   (let* ((info (file-name-sans-extension (howm-view-item-basename item)))
          (line (howm-view-item-summary item)))
-    (howm-menu-list-format info line item)))
+    (howm-menu-list-format info line item list-format)))
 
-(defun howm-menu-list-format (info line item)
-  (let ((s (format howm-menu-list-format info line)))
+(defun howm-menu-list-format (info line item &optional list-format)
+  (let ((s (format (or list-format howm-menu-list-format) info line)))
     (howm-menu-list-put-item s item)
     s))
 
@@ -798,7 +813,8 @@ TITLE-FORMAT is a format string for class title."
                         (is (howm-with-reminder-setting
                               (howm-todo-insert-separators
                                item-list
-                               howm-menu-reminder-separators))))
+                               howm-menu-reminder-separators
+                               t))))
                    (concat (format tform k)
                            (howm-menu-general (format "reminder(%s)" k) 'todo
                                               is))))
@@ -852,7 +868,8 @@ If you don't like misc. category, try
 (defun howm-menu-copy-skel (contents)
   (let ((menu-file (or howm-menu-file
                        (expand-file-name "0000-00-00-000000.howm"
-                                         howm-directory))))
+                                         howm-directory)))
+        (r "^="))
     (if (file-exists-p menu-file)
         ;; I have no courage to erase existing file.
         (progn
@@ -861,6 +878,9 @@ If you don't like misc. category, try
       (progn
         (find-file menu-file)
         (insert contents)
+        (goto-char (point-min))
+        (while (re-search-forward r nil t)
+          (replace-match howm-view-title-header nil nil))
         (save-buffer)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -916,9 +936,4 @@ If you don't like misc. category, try
 ;;   (put-text-property beg end 'intangible t)
   )
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; provide
-
-(provide 'howm-menu)
-
 ;;; howm-menu.el ends here
