@@ -1,7 +1,7 @@
 ;;; howm-misc.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2002, 2003, 2004, 2005, 2006
+;;; Copyright (c) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-misc.el,v 1.72 2006/05/01 15:07:05 hira Exp $
+;;; $Id: howm-misc.el,v 1.88 2009-02-05 15:00:55 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -19,7 +19,8 @@
 ;;; USA.
 ;;;--------------------------------------------------------------------
 
-(require 'howm-common)
+(provide 'howm-misc)
+(require 'howm)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Misc.
@@ -69,7 +70,7 @@
 ;;     ;; fix me
 ;;     (emacs-lisp-mode
 ;;      . (
-;;         (howm-keyword-format . "(def[a-z]+ +%s[ \t\r\n]")
+;;         (howm-keyword-format . "(def[a-z*]+ +%s[ \t\r\n]")
 ;;         (howm-keyword-regexp-format . "%s")
 ;;         (howm-keyword-regexp . "(\\(def[a-z]+\\) +'?\\([-+=*/_~!@$%^&:<>{}?a-zA-Z0-9]+\\)") ;; ' for (defalias 'xxx ...)
 ;;         (howm-keyword-regexp-hilit-pos . 1)
@@ -330,7 +331,7 @@ and replace a sub-expression, e.g.
         (apply #'concat (nreverse matches)))))
   )
 
-(defvar howm-kill-all-enable-force nil)
+(howm-defvar-risky howm-kill-all-enable-force nil)
 (defun howm-kill-all (&optional force-p)
   "Kill all buffers which is howm-mode and unmodified."
   (interactive "P")
@@ -394,18 +395,20 @@ and replace a sub-expression, e.g.
           (member buf keyword-bufs)))))
 
 (defun howm-mode-add-font-lock ()
+  (cheat-font-lock-append-keywords (howm-mode-add-font-lock-internal)))
+
+(defun howm-mode-add-font-lock-internal ()
   (when howm-use-color
-    (let ((ks `(,@howm-user-font-lock-keywords
-                (,howm-view-title-regexp
-                  (0 howm-mode-title-face prepend))
-                (,howm-keyword-regexp
-                 (,howm-keyword-regexp-hilit-pos howm-mode-keyword-face prepend))
-                (,howm-ref-regexp
-                 (,howm-ref-regexp-hilit-pos howm-mode-ref-face prepend))
-                (,howm-wiki-regexp
-                 (,howm-wiki-regexp-pos howm-mode-wiki-face prepend))
-                )))
-      (cheat-font-lock-append-keywords ks))))
+    `(,@howm-user-font-lock-keywords
+      (,howm-view-title-regexp
+       (0 howm-mode-title-face prepend))
+      (,howm-keyword-regexp
+       (,howm-keyword-regexp-hilit-pos howm-mode-keyword-face prepend))
+      (,howm-ref-regexp
+       (,howm-ref-regexp-hilit-pos howm-mode-ref-face prepend))
+      (,howm-wiki-regexp
+       (,howm-wiki-regexp-pos howm-mode-wiki-face prepend))
+      )))
 
 ;;; unofficial. may be removed if no one needs them.
 
@@ -583,7 +586,7 @@ and replace a sub-expression, e.g.
 (let ((m (make-sparse-keymap)))
   (define-key m "\C-c\C-c" 'howm-remember-submit)
   (define-key m "\C-c\C-k" 'howm-remember-discard)
-  (defvar howm-remember-mode-map m))
+  (howm-defvar-risky howm-remember-mode-map m))
 (defvar howm-remember-first-line-to-title nil)
 
 (defun howm-remember ()
@@ -639,6 +642,206 @@ and replace a sub-expression, e.g.
   (kill-buffer (current-buffer))
   (set-window-configuration howm-remember-wconf))
 
+;; Rename buffer
+;; 
+;; You can rename howm buffers based on their titles.
+;; Only buffer names in emacs are changed; file names are kept unchanged.
+;; 
+;; Add the next lines to your .emacs if you like this feature.
+;; (add-hook 'howm-mode-hook 'howm-mode-set-buffer-name)
+;; (add-hook 'after-save-hook 'howm-mode-set-buffer-name)
+;; 
+;; The original idea and code are given by Mielke-san (peter at exegenix.com).
+;; http://lists.sourceforge.jp/mailman/archives/howm-eng/2006/000020.html
+;; thx!
+
+(defvar howm-buffer-name-limit 20)
+(defvar howm-buffer-name-total-limit howm-buffer-name-limit)
+(defvar howm-buffer-name-format "%s"
+  "Buffer name format for `howm-mode-set-buffer-name'.
+For example, buffer name is _ABC_ if title is ABC and the value of
+this variable is \"_%s_\".")
+
+(defun howm-truncate-string (string limit &optional dots-str)
+  "Truncate STRING if it is longer than LIMIT.
+For example, \"1234567...\" is returned if string is \"123456789012\"
+and limit is 10.
+When DOTS-STR is non-nil, it is used instead of \"...\"."
+  (let ((dots (or dots-str "...")))
+    (when (> (length dots) limit)
+      (setq dots (substring dots 0 limit)))
+    (if (> (length string) limit)
+        (concat (substring string 0 (- limit (length dots)))
+                dots)
+      string)))
+
+(defun howm-set-buffer-name-from-title (checker title-regexp title-regexp-pos)
+  "Set the buffer name to the title(s) of the file."
+  (when (funcall checker)
+    (save-excursion
+      (goto-char 0)
+      (let ((titles nil))
+        (while (re-search-forward title-regexp nil t)
+          (setq titles
+                (cons (match-string-no-properties title-regexp-pos)
+                      titles))) 
+        (let ((name0 (mapconcat  
+                      (lambda (s)
+                        (howm-truncate-string s howm-buffer-name-limit))
+                      (reverse (howm-cl-remove-if (lambda (s) (string= s ""))
+                                                  titles))
+                      "/")))
+          (when (not (string= name0 "")) ;; exclude "no title" case
+            (let ((name (format howm-buffer-name-format
+                                (howm-truncate-string
+                                 name0
+                                 howm-buffer-name-total-limit))))
+              (rename-buffer name t))))))))
+
+(defun howm-mode-set-buffer-name ()
+  (howm-set-buffer-name-from-title (lambda ()
+                                     (and howm-mode (buffer-file-name)))
+                                   howm-view-title-regexp
+                                   howm-view-title-regexp-pos))
+
+;; memoize: used in howm-bayesian-set
+
+(defun howm-memoize-put (fname value)
+  (put fname 'howm-memoize value))
+(defun howm-memoize-get (fname)
+  (get fname 'howm-memoize))
+
+(defun howm-memoize-call (fname func args)
+  (let* ((p (assoc args (howm-memoize-get fname))))
+    (if p
+        (progn
+;;           (message "hit %s" p)
+          (cdr p))
+      (let ((r (apply func args)))
+        ;; We need to get it again because func can change memory.
+        (howm-memoize-put fname `((,args . ,r) ,@(howm-memoize-get fname)))
+        r))))
+
+(defun howm-memoize-reset (fname)
+  (howm-memoize-put fname nil))
+
+(put 'howm-defun-memoize 'lisp-indent-hook 2)
+(defmacro howm-defun-memoize (fname args &rest body)
+  `(progn
+     (howm-memoize-reset ',fname)
+     (defun ,fname ,args
+       "Function generated by `howm-defun-memoize'"
+       (howm-memoize-call ',fname (lambda ,args ,@body) (list ,@args)))))
+
+;; ;; test
+;; (howm-memoize-reset 'fib)
+;; (howm-defun-memoize fib (n) (if (<= n 1) 1 (+ (fib (- n 1)) (fib (- n 2)))))
+;; (fib 5)
+;; (howm-memoize-get 'fib)
+
+;; Bayesian set
+;; 
+;; "M-x howm-bayesian-set RET lisp scheme haskell RET" to estimate
+;; related keywords with lisp, scheme, and haskell.
+;; If you are lucky, you may find ruby, elisp, gauche, etc.
+;; in estimated candidates.
+;; 
+;; (ref.)
+;; Zoubin Ghahramani and Katherine Heller: "Bayesian Sets",
+;; Advances in Neural Information Processing Systems,
+;; Vol. 18, pp. 435-442, MIT Press, 2006.
+;; http://books.nips.cc/nips18.html
+;; http://books.nips.cc/papers/files/nips18/NIPS2005_0712.pdf
+
+(defun howm-bset-nodup (f &rest args)
+  (howm-cl-remove-duplicates (apply f args) :test #'equal))
+(defun howm-bset-mapcar (func lis)
+  (howm-bset-nodup #'mapcar func lis))
+(defun howm-bset-mapcan (func lis)
+  (howm-bset-nodup (lambda (f z) (apply #'append (mapcar f z)))
+                   func lis))
+
+(defun howm-bset-message (&rest args)
+  (let (message-log-max) ;; prevent it from being logged
+    (apply #'message args)))
+
+(defun howm-bset-matched-files (query)
+;;   (howm-bset-message "Finding files for query (%s)..." query)
+  (howm-bset-mapcar #'howm-item-name
+                    (howm-view-search-folder-items query (howm-folder)
+                                                   nil t)))
+
+(howm-defun-memoize howm-bset-keywords-in-file* (file keyword-list)
+;;   (howm-bset-message "Finding keywords in file (%s)..." file)
+  (with-temp-buffer
+    (insert-file-contents file)
+    (howm-keyword-for-goto keyword-list)))
+
+(defun howm-bset-keywords-in-file (file)
+  (howm-bset-keywords-in-file* file nil))
+
+(defun howm-bset-candidate-keywords (query-list)
+;;   (howm-bset-message "Collecting keywords...")
+  (let ((files (howm-bset-mapcan #'howm-bset-matched-files
+                                 query-list)))
+    (howm-bset-mapcan (lambda (f)
+                        (howm-bset-message "Collecting keywords in file (%s)..."
+                                           f)
+                        (howm-bset-keywords-in-file f))
+                      files)))
+
+(howm-defun-memoize howm-bset-file-score (file query-list
+                                               coef number-of-all-keywords)
+;;   (howm-bset-message "Scoring file (%s)..." file)
+  (let* ((m (/ (length (howm-bset-keywords-in-file file))
+               (float number-of-all-keywords)))
+         (a (* coef m))
+         (b (* coef (- 1 m)))
+         (s (length (howm-bset-keywords-in-file* file query-list)))
+         (a2 (+ a s))
+         (b2 (+ b (- (length query-list) s))))
+    ;; log{(a2/a) * (b/b2)}
+    (- (- (log a2) (log a)) (- (log b2) (log b)))))
+
+(howm-defun-memoize howm-bset-keyword-score (keyword query-list
+                                                     coef
+                                                     number-of-all-keywords)
+  (howm-bset-message "Scoring keyword (%s)..." keyword)
+  (apply #'+
+         (mapcar (lambda (file)
+                   (howm-bset-file-score file query-list coef
+                                         number-of-all-keywords))
+                 (howm-bset-matched-files keyword))))
+
+(defun howm-bset-reset ()
+  (mapc #'howm-memoize-reset '(howm-bset-file-score
+                               howm-bset-keyword-score
+                               howm-bset-keywords-in-file*)))
+
+(defun howm-bset (query-list)
+  (howm-bset-reset)
+  (unwind-protect
+      (let ((n (length (howm-keyword-list)))
+            (c 2.0)) ;; heuristic value
+        (sort (copy-sequence (howm-bset-candidate-keywords query-list))
+              (lambda (k1 k2)
+                (apply #'>
+                       (mapcar (lambda (k)
+                                 (howm-bset-keyword-score k query-list c n))
+                               (list k1 k2))))))
+    (howm-bset-reset)))
+
+(defun howm-bayesian-set (query-str)
+  (interactive "sQueries: ")
+  (switch-to-buffer (get-buffer-create "*howm-bayesian-set*"))
+  (howm-rewrite-read-only-buffer
+    (insert (mapconcat #'identity
+                       (howm-bset (split-string query-str))
+                       "\n"))
+    (howm-mode 1))
+  (goto-char (point-min))
+  (howm-bset-message "Done."))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Fellowship
 
@@ -761,15 +964,21 @@ and replace a sub-expression, e.g.
       ad-do-it
     (howm-auto-save-buffers-dispose)))
 (defun howm-basic-save-buffer ()
-  (basic-save-buffer)
-  ;; Okuyama's auto-save-buffers modifies write-region temporally.
+  "Silent version of `basic-save-buffer' without \"Wrote ...\" message."
+  (let ((original-write-region (symbol-function 'write-region)))
+    ;; make silent `write-region', which doesn't say "Wrote ...".
+    ;; I borrowed the idea from Okuyama's auto-save-buffers. thx.
+    ;; http://homepage3.nifty.com/oatu/emacs/misc.html
+    (flet ((write-region (start end filename
+                                &optional append visit lockname must)
+                         (funcall original-write-region
+                                  start end filename append
+                                  'dont-say-wrote-foobar
+                                  lockname must)))
+      (basic-save-buffer)))
   ;; As a side effect, basic-save-buffer does not update buffer-modified-p.
-  ;; http://homepage3.nifty.com/oatu/emacs/misc.html
-  (let ((in-auto-save-buffers (eq (symbol-function 'write-region)
-                                  'auto-save-buffers-write-region)))
-    (when in-auto-save-buffers
-      (set-visited-file-modtime)
-      (set-buffer-modified-p nil))))
+  (set-visited-file-modtime)
+  (set-buffer-modified-p nil))
 
 ;; howm on ChangeLog Memo
 (defun howm-setup-change-log ()
@@ -829,6 +1038,7 @@ and replace a sub-expression, e.g.
 ;; Japanese is assumed at now.
 
 (defun howm-test ()
+  "Show bug report template for howm."
   (howm-set-lang)
   (howm-bug-report))
 
@@ -865,6 +1075,16 @@ and replace a sub-expression, e.g.
               ("howm" . ,(howm-version-long))
               ,@(honest-report-version-assoc)
               ))
+    (when (eq howm-view-use-grep t)
+      (insert
+       (format "grep: %s - %s\n"
+               (howm-cl-mapcan (lambda (d)
+                                 (let ((f (expand-file-name
+                                           howm-view-grep-command d)))
+                                   (and (file-executable-p f)
+                                        (list f))))
+                               exec-path)
+               (car (howm-call-process "grep" '("--version"))))))
     (when show-sym
       (goto-char (point-max))
       (insert "\n(List of variables)\n")
@@ -878,12 +1098,6 @@ and replace a sub-expression, e.g.
           (howm-make-file-p)
           (howm-test-p)))
 
-(defun howm-emacs-build-time ()
-  (if (stringp emacs-build-time)
-      emacs-build-time  ;; xemacs
-    (format-time-string "%Y-%m-%d"
-                        emacs-build-time)))
-
 (defun howm-symbols-desc (&optional max-desc-len)
   (when (null max-desc-len)
     (setq max-desc-len 50))
@@ -910,6 +1124,7 @@ and replace a sub-expression, e.g.
                                 riffle
                                 gfunc
                                 illusion
+                                honest-report
                                 )
   "List of features which are required for, and distributed with, howm itself.")
 
@@ -949,8 +1164,4 @@ and replace a sub-expression, e.g.
     (honest-report header footer)
     (message "Please copy this buffer to your report.")))
 
-;;;
-
-(provide 'howm-misc)
-
 ;;; howm-misc.el ends here
