@@ -1,7 +1,7 @@
 ;;; howm-vars.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2005, 2006
+;;; Copyright (c) 2005, 2006, 2007, 2008, 2009
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-vars.el,v 1.31 2006/05/15 13:34:32 hira Exp $
+;;; $Id: howm-vars.el,v 1.49 2009-12-31 12:05:50 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -19,9 +19,35 @@
 ;;; USA.
 ;;--------------------------------------------------------------------
 
+(require 'howm-cl)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Util
 
+(defmacro howm-define-risky-command (risky orig)
+  "Define a macro RISKY which is risky-version of ORIG."
+  (let* ((gsymbol (howm-cl-gensym))
+         (gargs (howm-cl-gensym))
+         (docstring (format "Do `%s' and set risky-local-variable property."
+                            orig)))
+    `(progn
+       (put ',risky 'lisp-indent-hook 'defun)
+       (defmacro ,risky
+         (,gsymbol &rest ,gargs)
+         ,docstring
+         (let ((command ',orig)
+               (symbol ,gsymbol)
+               (args ,gargs))
+           `(progn
+              ;; (,',orig ...) doesn't work.
+              ;; So I need to bind temporal variables outside nested backquote.
+              (,command ,symbol ,@args)
+              (put ',symbol 'risky-local-variable t)))))))
+
+(howm-define-risky-command howm-defvar-risky defvar)
+(howm-define-risky-command howm-defcustom-risky defcustom)
+(howm-define-risky-command howm-defconst-risky defconst)
+
 ;; ;; Should I use this?
 ;; (defmacro howm-boundp-q (var)
 ;;   `(prog1
@@ -42,7 +68,7 @@ This is cheat to avoid warning while byte-compilation.
 Byte-compiler says \"not known to be defined\" even for codes like
   (if (fboundp 'foo) (foo bar)).
 
-(macroexpand '(howm-funcall-if-defined (migemo-get-pattern roma) nil))
+\(macroexpand '(howm-funcall-if-defined (migemo-get-pattern roma) nil))
 ==> (if (fboundp 'migemo-get-pattern)
         (let ((howm-funcall-if-defined-f 'migemo-get-pattern))
           (funcall howm-funcall-if-defined-f roma))
@@ -78,7 +104,7 @@ Byte-compiler says \"not known to be defined\" even for codes like
   "Names of files and directories."
   :group 'howm)
 
-(defcustom howm-directory "~/howm/"
+(howm-defcustom-risky howm-directory "~/howm/"
   "*All files under this directory are scanned recursively."
   :type 'directory
   :group 'howm-files)
@@ -96,7 +122,7 @@ to its own file. You must guarantee (string< oldfile newfile)."
   :group 'howm-efficiency
   :group 'howm-files)
 
-(defcustom howm-keyword-file "~/.howm-keys"
+(howm-defcustom-risky howm-keyword-file "~/.howm-keys"
   "*Keywords (WikiNames) are stored in this file."
   :type 'file
   :group 'howm-files)
@@ -124,6 +150,8 @@ max-specpdl-size must be increased from the default value 600.
 Otherwise, an error occurs both in byte-compilation and in run time.
 To avoid such troubles, this variable is prepared as a fixed string.")
 
+(defvar howm-excluded-dirs '("RCS" "CVS" ".svn" ".git" "_darcs"))
+
 (defvar howm-excluded-file-regexp-common-list
   (list "[~#]$"
         "\\.\\(bak\\|elc\\|gz\\|aux\\|toc\\|idx\\|dvi\\)$"
@@ -134,22 +162,21 @@ To avoid such troubles, this variable is prepared as a fixed string.")
       "[/\\\\]" ;; / or \ for win
     "/")) ;; / otherwise
 (let ((dir-head (concat "\\(^\\|" howm-excluded-file-regexp-dir-sep "\\)"))
-      (cvs (concat "CVS" howm-excluded-file-regexp-dir-sep)))
+      (excluded-dirs (concat (regexp-opt howm-excluded-dirs t)
+                   howm-excluded-file-regexp-dir-sep)))
   (defvar howm-excluded-file-regexp-dots-ok
     (mapconcat #'identity
-               `(,(concat dir-head cvs) ;; "\\(^\\|/\\)CVS/"
+               `(,(concat dir-head excluded-dirs)
                  "^[.][.]"
                  ,@howm-excluded-file-regexp-common-list)
                "\\|"))
   (defvar howm-excluded-file-regexp-dots-ng
     (mapconcat #'identity
-               `(,(concat dir-head "\\([.]\\|" cvs "\\)")
-                 ;; "\\(^\\|/\\)\\([.]\\|CVS/\\)"
+               `(,(concat dir-head "\\([.]\\|" excluded-dirs "\\)")
                  ,@howm-excluded-file-regexp-common-list)
                "\\|")))
-;;   "\\(^\\|/\\)\\([.]\\|CVS/\\)\\|[~#]$\\|\\.\\(bak\\|elc\\|gz\\|aux\\|toc\\|idx\\|dvi\\|jpg\\|gif\\|png\\)$"
 
-(defcustom howm-excluded-file-regexp howm-excluded-file-regexp-dots-ng
+(howm-defcustom-risky howm-excluded-file-regexp howm-excluded-file-regexp-dots-ng
   "Regexp for excluded files.
 It is checked for relative paths from howm-directory and howm-search-path.
 A file is excluded iff this regexp matches with all the relative paths."
@@ -178,7 +205,7 @@ A file is excluded iff this regexp matches with all the relative paths."
   :type '(radio (const en) (const ja))
   :group 'howm-menu)
 
-(defcustom howm-menu-file nil
+(howm-defcustom-risky howm-menu-file nil
   "*Specify menu file explicitly, or set as nil to search every time."
   :type '(radio (const :tag "Search every time" nil)
                 (const "0000-00-00-000000.howm")
@@ -332,6 +359,7 @@ Be careful that you cannot undo the result of action-lock after kill-buffer."
 (defvar howm-huge- 66666)
 (defvar howm-huge 77777)
 (defvar howm-huge+ 88888)
+(defvar howm-huge++ 99999)
 
 (defcustom howm-menu-todo-priority (- howm-huge+)
   "*Limit priority for elimination of reminders in menu."
@@ -341,7 +369,8 @@ Be careful that you cannot undo the result of action-lock after kill-buffer."
   :group 'howm-menu-reminder)
 
 (defcustom howm-todo-priority-done-bottom (- howm-huge+)
-  "*Priority of done reminder."
+  "*Base priority of done reminder.
+<priority of done reminder> = <this value> + <days from specified date>"
   :type `(radio (const :tag "Deeper than sleeping reminders" ,(- howm-huge+))
                 (const :tag "Shallower than sleeping reminders"
                        ,(- howm-huge-))
@@ -376,7 +405,7 @@ When it is nil, `howm-view-title-regexp' is used."
   "Style of list view."
   :group 'howm)
 
-(defcustom howm-view-contents-limit 10000
+(defcustom howm-view-contents-limit nil
   "*Max length for howm-view-contents. Nil means no limit."
   :type '(radio (const :tag "No limit" nil)
                 integer)
@@ -405,14 +434,27 @@ When it is nil, `howm-view-title-regexp' is used."
   "Sorting and filtering of matched entries."
   :group 'howm-list)
 
-(defcustom howm-list-normalizer 'howm-view-sort-by-mtime
-  "*Default method to list matched notes."
-  :type '(radio (function-item :tag "Sort by edit-time" howm-view-sort-by-mtime)
+(howm-defcustom-risky howm-list-normalizer nil
+  "*Obsolete. Use `howm-normalizer' insteadly."
+  :type '(radio (const :tag "Off (strongly recommended)" nil)
+                (function-item :tag "Sort by edit-time"
+                               howm-view-sort-by-mtime)
                 (function-item :tag "Sort by create-time"
                                howm-view-sort-by-reverse-date)
                 function)
   :group 'howm-sort)
 
+(howm-defcustom-risky howm-normalizer 'howm-sort-items-by-mtime
+  "*Default method to list matched notes.
+For backward compatibility, this value is overridden
+if `howm-list-normalizer' is non-nil."
+  :type '(radio (function-item :tag "Sort by edit-time"
+                               howm-sort-items-by-mtime)
+                (function-item :tag "Sort by create-time"
+                               howm-sort-items-by-reverse-date)
+                function)
+  :group 'howm-sort)
+
 (defcustom howm-list-prefer-word nil
   "*Matches to whole word are listed first in summary buffer."
   :type 'boolean
@@ -445,7 +487,7 @@ When it is nil, `howm-view-title-regexp' is used."
                     howm-action-lock-date-search
                     ))))
 
-(defcustom howm-list-title
+(howm-defcustom-risky howm-list-title
   '(
     howm-list-all
     howm-list-recent
@@ -481,7 +523,8 @@ When it is nil, `howm-view-title-regexp' is used."
   :group 'howm-title)
 
 (defcustom howm-list-title-undo t
-  "*Non-nil if `howm-list-title' should toggle whether title is shown or not."
+  "*Non-nil if `howm-list-toggle-title' should toggle whether title is shown
+or not."
   :type 'boolean
   :group 'howm-efficiency
   :group 'howm-title)
@@ -510,13 +553,13 @@ When it is nil, `howm-view-title-regexp' is used."
                 string)
   :group 'howm-list-bufwin)
 
-(defcustom howm-view-summary-persistent t
+(howm-defcustom-risky howm-view-summary-persistent t
   "*If non-nil, keep summary buffer on howm-view-summary-open by default.
 If it is a function, the evaluated value is used instead of itself."
   :type 'boolean
   :group 'howm-list-bufwin)
 
-(defcustom howm-view-contents-persistent t
+(howm-defcustom-risky howm-view-contents-persistent t
   "*If non-nil, keep contents buffer on howm-view-contents-open by default.
 If it is a function, the evaluated value is used instead of itself."
   :type 'boolean
@@ -534,6 +577,16 @@ If it is a function, the evaluated value is used instead of itself."
   :type 'boolean
   :group 'howm-search)
 
+(defcustom howm-check-word-break nil
+  "*Non-nil if come-from keywords within a word should not linked.
+When the value is a string (regexp), word breaks are checked
+only for matched keywords. "
+  :type '(radio (const :tag "Always" t)
+                (const :tag "Never" nil)
+                (const :tag "ASCII only" "^[[:ascii:]]+$")
+                string)
+  :group 'howm-search)
+
 (defcustom howm-view-update-search-ring nil
   "*Non-nil if search-ring should be updated in howm search."
   :type 'boolean
@@ -545,7 +598,7 @@ If it is a function, the evaluated value is used instead of itself."
   :group 'howm-devel
   :group 'howm-search)
 
-(defcustom howm-history-file "~/.howm-history"
+(howm-defcustom-risky howm-history-file "~/.howm-history"
   "*Search history is recorded to that file."
   :type 'file
   :group 'howm-files
@@ -585,54 +638,64 @@ If it is nil, alias of come-from keyword is disabled."
   :group 'howm-efficiency
   :group 'howm-search)
 
-(defcustom howm-view-use-grep nil
+(howm-defcustom-risky howm-view-use-grep nil
   "*If non-nil, use external grep command for search.
 Performance must be improved greatly if you set this.
-When the value is elisp function, it is used instead of `howm-view-fake-grep'."
+When the value is elisp function, it is used instead of `howm-fake-grep'."
   :type '(radio (const :tag "On" t)
                 (const :tag "Off" nil)
                 function)
   :group 'howm-grep)
 
 ;; These variables should be renamed: howm-view-xxx ==> howm-xxx.
-(defcustom howm-view-grep-command "grep"
+(howm-defcustom-risky howm-view-grep-command "grep"
   "*Command name for grep."
   :type 'string
   :group 'howm-grep)
-(defvar howm-view-fgrep-command nil
+(howm-defvar-risky howm-view-fgrep-command nil
   "*Command name for fgrep.
 This variable is obsolete and may be removed in future.")
-(defcustom howm-view-grep-option "-Hnr"
+(defvar howm-view-grep-default-option
+  (concat "-Hnr "
+          (mapconcat (lambda (d) (concat "--exclude-dir=" d))
+                     howm-excluded-dirs " ")))
+(howm-defcustom-risky howm-view-grep-option howm-view-grep-default-option
   "*Common grep option for howm."
-  :type 'string
+  :type `(radio (const :tag "scan all files"
+                       ,howm-view-grep-default-option)
+                (const :tag "scan *.howm only"
+                       ,(concat howm-view-grep-default-option
+                                " --include=*.howm"))
+                string)
+  :group 'howm-efficiency
   :group 'howm-grep)
-(defcustom howm-view-grep-extended-option "-E"
+(howm-defcustom-risky howm-view-grep-extended-option "-E"
   "*Grep option for extended regular expression."
   :type 'string
   :group 'howm-grep)
-(defcustom howm-view-grep-fixed-option "-F"
+(howm-defcustom-risky howm-view-grep-fixed-option "-F"
   "*Grep option to search fixed strings."
   :type 'string
   :group 'howm-grep)
-(defcustom howm-view-grep-ignore-case-option "-i"
+(howm-defcustom-risky howm-view-grep-ignore-case-option "-i"
   "*Grep option for ignoring case distinctions."
   :type 'string
   :group 'howm-grep)
-(defcustom howm-view-grep-expr-option "-e"
+(howm-defcustom-risky howm-view-grep-expr-option "-e"
   "*Grep option for pattern."
   :type 'string
   :group 'howm-grep)
-(defcustom howm-view-grep-file-stdin-option "-f -"
+(howm-defcustom-risky howm-view-grep-file-stdin-option "-f -"
   "*Grep option for receiving patterns from standard input.
 If this is nil, pattern is received as command line argument."
   :type '(radio (const :tag "Off" nil)
                 string)
   :group 'howm-grep)
-;; (defvar howm-view-grep-command "egrep")
-;; (defvar howm-view-fgrep-command "fgrep")
-;; (defvar howm-view-grep-extended-option nil)
-;; (defvar howm-view-grep-fixed-option nil)
-;; (defvar howm-view-grep-file-stdin-option "-f -")
+
+(howm-defcustom-risky howm-command-length-limit 10000
+  "*Maximum length of command line for call-process."
+  :type 'integer
+  :group 'howm-grep)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Misc
@@ -698,13 +761,17 @@ rather than append or prepend."
 
 ;; These variables should be renamed.
 
-(defcustom howm-view-external-viewer-assoc nil
+(howm-defcustom-risky howm-view-external-viewer-assoc nil
   "List of viewer specifications.
 Each specification must be a cons pair of type and format.
 Type is a regular expression of file names.
 Format is a command string in which %s is replaced with file name.
 This setting is prior to mailcap.
 
+This variable is marked as a risky local variable
+because `howm-viewer-dispatchers' `howm-viewer-indicator'
+and `howm-viewer-type' accept functions instead of format strings.
+
 Example:
   (setq howm-view-external-viewer-assoc
         '(
@@ -835,6 +902,11 @@ If the value is t, it means 'always'."
     (t ()))
   "*Face for deadline."
   :group 'howm-faces)
+(defface howm-reminder-late-deadline-face
+  '((((class color)) (:background "red" :foreground "black"))
+    (t ()))
+  "*Face for late deadline."
+  :group 'howm-faces)
 (defface howm-reminder-schedule-face
   '((((class color) (background light)) (:foreground "dark green"))
     (((class color) (background dark)) (:foreground "green"))
@@ -891,6 +963,8 @@ If the value is t, it means 'always'."
   "*Face for defer.")
 (defvar howm-reminder-deadline-face 'howm-reminder-deadline-face
   "*Face for deadline.")
+(defvar howm-reminder-late-deadline-face 'howm-reminder-late-deadline-face
+  "*Face for late deadline.")
 (defvar howm-reminder-schedule-face 'howm-reminder-schedule-face
   "*Face for schedule.")
 (defvar howm-reminder-done-face     'howm-reminder-done-face
@@ -990,7 +1064,7 @@ If the value is t, it means 'always'."
   :type 'boolean
   :group 'howm-experimental)
 
-(defcustom howm-user-font-lock-keywords nil
+(howm-defcustom-risky howm-user-font-lock-keywords nil
   "Font lock keywords for all howm-related buffers.
 See help of `font-lock-keywords' for details."
 ;;   :type '(repeat (radio (cons regexp (list (const quote) face))
@@ -1011,7 +1085,18 @@ This option is prepared for `howm-menu-reminder'."
                           (nil . ,sep)))
                   (alist :key-type
                          (radio number
-                                (const :tag "Between schedule and todo"))
+                                (const :tag "Between schedule and todo" nil))
+                         :value-type string))
+    :group 'howm-experimental)
+  (defcustom howm-todo-separators nil
+    "Assoc list to specify positions and strings of separators in todo buffer.
+For each element, car is priority and cdr is separator string.
+If car is nil, it means the boarder between active and sleeping reminders."
+    :type `(radio (const :tag "No separators" nil)
+                  (const :tag "Standard separators"
+                         ((0 . ,sep)
+                          (nil . ,sep)))
+                  (alist :key-type number
                          :value-type string))
     :group 'howm-experimental))
 
@@ -1031,10 +1116,11 @@ Example:
   :group 'howm-experimental
   )
 
-(defcustom howm-view-dired-keep-cursor nil
-  "*Non nil if `howm-view-dired' should keep cursor position."
-  :type 'boolean
-  :group 'howm-experimental)
+(defcustom howm-action-lock-forward-fuzziness 0
+  "*Maximum lines of permitted inconsistency for `howm-action-lock-forward'."
+  :type 'integer
+  :group 'howm-experimental
+  )
 
 ;;;
 
