@@ -1,7 +1,7 @@
 ;;; illusion.el --- load, edit, and submit something which is not pure file
-;;; Copyright (c) 2005, 2006
+;;; Copyright (c) 2005, 2006, 2007, 2008, 2009
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: illusion.el,v 1.5 2006/03/22 13:29:25 hira Exp $
+;;; $Id: illusion.el,v 1.11 2009-02-05 15:00:55 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
   "Value of this variable is called when `illusion-submit' is executed.
 It must return non-nil value for successful case.")
 (make-variable-buffer-local 'illusion-submit-func)
+(put 'illusion-submit-func 'risky-local-variable t)
 
 (defun illusion-submit ()
   (interactive)
@@ -90,7 +91,7 @@ key	binding
 (defun yarot13-insert-file-contents (file)
   (if (file-exists-p file)
       (let ((s (with-temp-buffer
-                 (insert-file-contents file)
+                 (howm-insert-file-contents file)
                  (yarot13-rotate-buffer)
                  (buffer-string))))
         (insert s))
